@@ -1,22 +1,25 @@
 from .config import __version__, __author__, __description__
-from .docs import (
-    ThermoDB,
-    CompBuilder,
-    TableData,
+from .core import (
     TableEquation,
-    TableMatrixData,
     TableMatrixEquation,
-    ManageData,
-    CustomRef
+    TableData,
+    TableMatrixData
 )
+from .docs import ThermoDB
+from .builder import CompBuilder
+from .loader import CustomRef
+from .manager import ManageData
 from .app import (
     init,
     ref,
     build_thermodb,
     load_thermodb,
+)
+from .thermodb import (
     build_component_thermodb,
     build_components_thermodb,
     build_component_thermodb_from_reference,
+    check_and_build_component_thermodb,
     ComponentThermoDB
 )
 
@@ -39,5 +42,6 @@ __all__ = [
     'ManageData',
     'CustomRef',
     'build_component_thermodb_from_reference',
+    'check_and_build_component_thermodb',
     'ComponentThermoDB'
 ]
