@@ -1,4 +1,5 @@
-# import packages/modules
+# import libs
+import logging
 import os
 import pandas as pd
 from typing import (
@@ -7,12 +8,15 @@ from typing import (
 )
 import glob
 # local
-from .managedata import ManageData
+from ..manager import ManageData
 from ..data import TableTypes
 from ..models import PayLoadType
-from .customref import CustomRef
+from ..loader import CustomRef
 from ..models import DataBookTableTypes
 
+# NOTE: logger
+logger = logging.getLogger(__name__)
+
 
 class TableReference(ManageData):
     """
@@ -356,6 +360,8 @@ class TableReference(ManageData):
             column name
         lookup : str
             value to look up for
+        query : bool, optional
+            if True, then use query method, by default False
 
         Returns
         -------
@@ -372,7 +378,7 @@ class TableReference(ManageData):
             # check tb_type
             if (
                 tb_type == TableTypes.DATA.value or
-                    tb_type == TableTypes.EQUATIONS.value
+                tb_type == TableTypes.EQUATIONS.value
             ):
                 # ! search table
                 df = self.search_table(
@@ -439,38 +445,64 @@ class TableReference(ManageData):
                 df_info = df.iloc[:2, :]
 
                 # SECTION: filter
-                if isinstance(column_name, str) and query is False:
+                if (
+                    isinstance(column_name, str) and
+                    query is False
+                ):  # ! search by column name
                     # NOTE: check query
                     # create filter
-                    # NOTE: check lookup
+                    # check lookup
                     if not isinstance(lookup, str):
                         raise ValueError(
-                            f"Lookup value must be a string for {databook_id} and {table_id}.")
+                            f"Lookup value must be a string for {databook_id} and {table_id}."
+                        )
 
                     df_filter = df[
                         df[column_name].str.lower() == lookup.lower()
                     ]
-                elif isinstance(column_name, str) and query is True:
+                elif (
+                    isinstance(column_name, str) and
+                    query is True
+                ):  # ! search by query
                     # NOTE: check query
                     # create filter
-                    df_filter = df.query(column_name)
+                    df_filter = df.query(column_name, engine='python')
 
                 elif (
                     isinstance(column_name, list) and
-                    isinstance(lookup, list)
-                ):
-                    # NOTE: check lookup
+                    isinstance(lookup, list) and
+                    query is False
+                ):  # ! search by list of column names
+                    # NOTE: check column names
+                    if len(column_name) != len(lookup):
+                        raise ValueError(
+                            f"Column name and lookup must have the same length for {databook_id} and {table_id}."
+                        )
+
+                    # SECTION: use query
                     # use query
                     _query = []
+
+                    # iterate through column names
                     for i in range(len(column_name)):
-                        _query.append(f'`{column_name[i]}` == "{lookup[i]}"')
+                        _query.append(
+                            f'{column_name[i]}.str.lower() == "{str(lookup[i]).lower()}"'
+                        )
+
                     # make query
-                    _query_set = ' & '.join(_query)
-                    # query
-                    df_filter = df.query(_query_set)
+                    _query_set = ' and '.join(_query)
+
+                    # NOTE: query
+                    df_filter = df.query(_query_set, engine='python')
+                else:
+                    raise ValueError(
+                        f"Column name and lookup formats are not valid for {databook_id} and {table_id}."
+                    )
 
                 # SECTION: combine dfs
                 result = pd.concat([df_info, df_filter])
+
+                # NOTE: check
                 if not df_filter.empty:
                     return result
                 else:
@@ -637,6 +669,9 @@ class TableReference(ManageData):
             records, if nan exists then converted to 0
         '''
         try:
+            # table type
+            tb_type = self.get_table_type(databook_id, table_id)
+
             # SECTION: check
             if matrix_tb:
                 df = self.search_matrix_table(
