@@ -19,18 +19,25 @@ from ..config import API_URL, __version__
 from ..api import Manage
 from ..utils import isNumber, uppercaseStringList
 from .tableref import TableReference
-from .transdata import TransData
-from .transmatrixdata import TransMatrixData
-from .managedata import ManageData
-from .tableequation import TableEquation
-from .tablematrixequation import TableMatrixEquation
-from .tabledata import TableData
-from .tablematrixdata import TableMatrixData
+# transformer
+from ..transformer import TransData
+from ..transformer import TransMatrixData
+from ..manager import ManageData
+# core
+from ..core import (
+    TableEquation,
+    TableMatrixEquation,
+    TableData,
+    TableMatrixData
+)
 from ..data import TableTypes
 from ..models import DataBookTableTypes, Component
 # web app
 from ..ui import Launcher
 
+# NOTE: logger
+logger = logging.getLogger(__name__)
+
 # SECTION: types
 ComponentSearch = Union[
     list[dict[str, str]],
@@ -334,9 +341,9 @@ class ThermoDB(ManageData):
                     for i, item in enumerate(tables):
                         # check
                         if isinstance(item, list):
-                            # table name
-                            tb_name = item[0]
-                            if tb_name == table.strip():
+                            # ! table name (case insensitive)
+                            tb_name = item[0].lower().strip()
+                            if tb_name == table.strip().lower():
                                 # zero-based id
                                 tb_id = i
                                 break
@@ -1204,17 +1211,36 @@ class ThermoDB(ManageData):
         Returns
         -------
         str | dict[str, str]
-            summary of the component availability
+            summary of the component availability as a string or dictionary in the specified format.
+
+            - 'databook_id': databook id,
+            - 'databook_name': 'Thermodynamic Properties of Pure Compounds',
+            - 'table_id': table id,
+            - 'table_name': 'Physical Properties of Pure Compounds',
+            - 'component_name': name of the component,
+            - 'availability': True or False
+
+        Notes
+        -----
+        1. component_name can be a string or a list of strings.
+            If a list is provided, it should contain the component name and state (e.g., ['Carbon dioxide', 'g']).
+        2. column_name can be a string or a list of strings.
+            If a list is provided, it should contain the column names to search in (e.g., ['Name', 'State']).
+        3. If query is provided, it will be used to search the dataframe directly.
+        4. If column_name is not provided, it defaults to 'Name'.
+        5. If query is not provided, the method will construct a query based on component_name and column_name.
+        6. The method returns the result in the specified format: 'dict', 'json', or 'str'.
         '''
         try:
-            # check search option
+            # NOTE: check search option
             if column_name is None:
                 column_name = 'Name'
 
-            # check
+            # NOTE: check
             if query:
                 column_name = column_name
 
+            # SECTION: find ids
             # find databook zero-based id (real)
             db, db_name, db_rid = self.find_databook(databook)
             # databook id (non-zero-based id)
@@ -1237,8 +1263,12 @@ class ThermoDB(ManageData):
                     pass
                 elif self.data_source == 'local':
                     res = self.check_component_local(
-                        component_name, databook_id, table_id,
-                        column_name, query=query)
+                        component_name=component_name,
+                        databook_id=databook_id,
+                        table_id=table_id,
+                        column_name=column_name,
+                        query=query
+                    )
                 else:
                     raise Exception('Data source error!')
             else:
@@ -1247,7 +1277,9 @@ class ThermoDB(ManageData):
             # res
             res_dict = {
                 'databook_id': databook_id,
+                'databook_name': db_name,
                 'table_id': table_id,
+                'table_name': tb_name,
                 'component_name': component_name,
                 'availability': res
             }
@@ -1267,6 +1299,142 @@ class ThermoDB(ManageData):
         except Exception as e:
             raise Exception(f"Component check error! {e}")
 
+    def is_component_available(
+        self,
+        component: Component,
+        databook: int | str,
+        table: int | str,
+        column_names: List[str] = ['Name', 'Formula'],
+        component_key: Literal['Name-State', 'Formula-State'] = 'Name-State',
+        res_format: Literal['dict', 'json', 'str'] = 'dict'
+    ) -> Union[str, dict[str, str]]:
+        '''
+        Check if a component is available in the specified databook and table. A component is defined as:
+        - name-state: carbon dioxide-g
+        - formula-state: CO2-g
+
+        Parameters
+        ----------
+        component : Component
+            The component to check.
+        databook : int | str
+            The databook id or name.
+        table : int | str
+            The table id or name.
+        column_names : List[str], optional
+            List of column names to search in, by default ['Name', 'Formula'].
+        component_key : Literal['Name-State', 'Formula-State'], optional
+            The key to use for identifying the component, by default 'Name-State'.
+        res_format : Literal['dict', 'json', 'str'], optional
+            The format of the returned result, by default 'json'.
+
+        Returns
+        -------
+        str | dict[str, str]
+            Summary of the component availability as a string or dictionary in the specified format.
+
+            - 'databook_id': databook id,
+            - 'databook_name': 'Thermodynamic Properties of Pure Compounds',
+            - 'table_id': table id,
+            - 'table_name': 'Physical Properties of Pure Compounds',
+            - 'component_name': name of the component,
+            - 'availability': True or False
+
+        Notes
+        -----
+        - Table should contain columns for 'Name', 'Formula', and 'State'. Otherwise an error will be raised.
+        '''
+        try:
+            # SECTION: Validate input
+            if not isinstance(component, Component):
+                raise ValueError(
+                    "Invalid component. Must be an instance of Component class.")
+            if (
+                not component.name and
+                not component.formula and
+                not component.state
+            ):
+                raise ValueError(
+                    "Component must have at least a name or a formula.")
+            if not column_names or not isinstance(column_names, list):
+                raise ValueError(
+                    "column_names must be a non-empty list of strings.")
+            if len(column_names) != 2:
+                raise ValueError(
+                    "column_names must contain exactly two elements: ['Name', 'Formula'].")
+
+            # SECTION: Check component_key validity
+            if component_key not in ['Name-State', 'Formula-State']:
+                raise ValueError(
+                    "Invalid component_key. Must be 'Name-State' or 'Formula-State'.")
+
+            # NOTE: create component id band query
+            component_id = None
+            query = None
+
+            # check
+            if component_key == 'Name-State':
+                # set
+                component_id = f"{component.name}-{component.state}"
+                # query
+                query = f'Name.str.lower() == "{component.name.lower()}" and State.str.lower() == "{component.state.lower()}"'
+            elif component_key == 'Formula-State':
+                # set
+                component_id = f"{component.formula}-{component.state}"
+                # query
+                query = f'Formula.str.lower() == "{component.formula.lower()}" and State.str.lower() == "{component.state.lower()}"'
+
+            if component_id is None:
+                raise ValueError("Component ID could not be determined.")
+
+            if query is None:
+                raise ValueError("Query could not be determined.")
+
+            # SECTION: find ids
+            # find databook zero-based id (real)
+            db, db_name, db_rid = self.find_databook(databook)
+            # databook id (non-zero-based id)
+            databook_id = db_rid + 1
+
+            # find table zero-based id
+            tb_id, tb_name = self.find_table(databook, table)
+            # table id (non-zero-based id)
+            table_id = tb_id + 1
+
+            # SECTION: Check if the component exists in the specified databook and table
+            availability = self.check_component_local(
+                component_name=component_id,
+                databook_id=databook_id,
+                table_id=table_id,
+                column_name=query,
+                query=True,
+            )
+
+            # res
+            res_dict = {
+                'databook_id': databook_id,
+                'databook_name': db_name,
+                'table_id': table_id,
+                'table_name': tb_name,
+                'component_name': component_id,
+                'availability': availability
+            }
+
+            # json
+            res_json = json.dumps(res_dict, indent=4)
+
+            # check
+            if res_format == 'json':
+                return res_json
+            elif res_format == 'dict':
+                return res_dict
+            elif res_format == 'str':
+                return res_json
+            else:
+                raise ValueError('Invalid res_format')
+        except Exception as e:
+            raise Exception(f"Error checking component availability: {e}")
+
     def check_component_api(
             self,
             component_name: str | list,
@@ -1368,6 +1536,11 @@ class ThermoDB(ManageData):
         -------
         object : bool
             component information
+
+        Notes
+        -----
+        - Return True if df is not empty
+        - Return False if df is empty
         '''
         try:
             # check databook_id and table_id are number or not
@@ -1380,38 +1553,19 @@ class ThermoDB(ManageData):
 
                 # NOTE: search
                 df = TableReferenceC.search_tables(
-                    databook_id,
-                    table_id,
-                    column_name,
-                    component_name,
+                    databook_id=databook_id,
+                    table_id=table_id,
+                    column_name=column_name,
+                    lookup=component_name,  # ! exact match
                     query=query
                 )
 
                 # NOTE: check availability
                 if len(df) > 0:
-                    # get databook
-                    databook_name = self.list_databooks(
-                        res_format='list'
-                    )[
-                        databook_id-1
-                    ]
-                    # get table
-                    # table_name = self.list_tables(databook=databook_id, res_format='list')[
-                    #     table_id-1][0]
-
-                    table_name_ = self.list_tables(
-                        databook=databook_id,
-                        res_format='list'
-                    )
-
-                    # heck
-                    if isinstance(table_name_, list):
-                        table_name = table_name_[table_id-1][0]
-
-                    # log
                     if verbose:
                         print(
-                            f"[{component_name}] available in [{table_name}] | [{databook_name}]")
+                            f"[{component_name}] available in [{table_id}] | [{databook_id}]"
+                        )
 
                     # res
                     return True
@@ -1419,12 +1573,16 @@ class ThermoDB(ManageData):
                     # log
                     if verbose:
                         print(f"{component_name} is not available.")
+
                     # res
                     return False
             else:
-                raise Exception("databook and table id required!")
+                logger.error(
+                    "Invalid input. Please check the input type (databook_id and table_id).")
+                return False
         except Exception as e:
-            raise Exception(f'Reading data error {e}')
+            logger.error(f'Checking data error {e}')
+            return False
 
     def get_component_data(
         self,
@@ -1604,14 +1762,18 @@ class ThermoDB(ManageData):
                     lookup = component_name
 
                 # NOTE: search
-                payload = TableReferenceC.make_payload(
-                    databook_id=databook_id,
-                    table_id=table_id,
-                    column_name=column_name,
-                    lookup=lookup,
-                    query=query,
-                    matrix_tb=matrix_tb
-                )
+                try:
+                    payload = TableReferenceC.make_payload(
+                        databook_id=databook_id,
+                        table_id=table_id,
+                        column_name=column_name,
+                        lookup=lookup,
+                        query=query,
+                        matrix_tb=matrix_tb
+                    )
+                except Exception as e:
+                    logging.error(f"Table search error {e}")
+                    payload = None
 
                 # NOTE: check availability
                 if payload:
@@ -1642,7 +1804,8 @@ class ThermoDB(ManageData):
         self,
         component_names: list[str],
         databook: int | str,
-        table: int | str
+        table: int | str,
+        **kwargs
     ) -> ThermoProperty:
         """
         Build a thermodynamic property including data, equation, matrix-data and matrix-equation.
@@ -1655,6 +1818,10 @@ class ThermoDB(ManageData):
             databook id or name
         table : int | str
             table id or name
+        **kwargs
+            Additional keyword arguments to pass to the specific build methods.
+            - component_state : str, optional
+                component state (e.g. 'g', 'l', 's')
 
         Returns
         -------
@@ -1666,31 +1833,49 @@ class ThermoDB(ManageData):
             - TableMatrixData
         """
         try:
-            # detect table type
+            # SECTION: extract kwargs
+            # LINK: column name
+            column_name = kwargs.get('column_name', None)
+            # LINK: query
+            query = kwargs.get('query', False)
+
+            # SECTION: detect table type
             tb_info_res_ = self.table_info(databook, table, res_format='dict')
 
-            # if
+            # SECTION: build thermo property
             if isinstance(tb_info_res_, dict):
                 # check
                 if tb_info_res_['Type'] == 'Equation':  # ! equation
                     # check
                     if len(component_names) > 1:
                         raise Exception('Only one component name required!')
+
+                    # component name set
+                    component_name_ = component_names[0]
+
                     # build equation
                     return self.build_equation(
-                        component_names[0],
-                        databook,
-                        table
+                        component_name=component_name_,
+                        databook=databook,
+                        table=table,
+                        column_name=column_name,
+                        query=query
                     )
                 elif tb_info_res_['Type'] == 'Data':  # ! data
                     # check
                     if len(component_names) > 1:
                         raise Exception('Only one component name required!')
+
+                    # component name set
+                    component_name_ = component_names[0]
+
                     # build data
                     return self.build_data(
-                        component_names[0],
-                        databook,
-                        table
+                        component_name=component_name_,
+                        databook=databook,
+                        table=table,
+                        column_name=column_name,
+                        query=query
                     )
                 elif tb_info_res_['Type'] == 'Matrix-Equation':  # ! matrix-equation
                     # check
@@ -1710,9 +1895,9 @@ class ThermoDB(ManageData):
                             'At least two component names required!')
                     # build matrix-data
                     return self.build_matrix_data(
-                        component_names,
-                        databook,
-                        table
+                        component_names=component_names,
+                        databook=databook,
+                        table=table
                     )
                 else:
                     raise Exception('No data/equation found!')
@@ -1726,8 +1911,10 @@ class ThermoDB(ManageData):
         components: List[Component],
         databook: int | str,
         table: int | str,
-        column_name: Optional[str | list[str]] = None,
-    ):
+        component_key: Literal[
+            'Name-State', 'Formula-State'
+        ] = 'Name-State',
+    ) -> ThermoProperty:
         '''
         Build thermo property for a component including data, equation, matrix-data and matrix-equation.
 
@@ -1741,9 +1928,24 @@ class ThermoDB(ManageData):
             databook id or name
         table : int | str
             table id or name
-        column_name : str | list, optional
-            column name (e.g. 'Name') | list as ['Name','State']
+        component_key : Literal['Name-State', 'Formula-State'], optional
+            The key to use for identifying the component, by default 'Name-State'.
 
+        Returns
+        -------
+        ThermoProperty
+            table object with data loaded
+            - TableEquation
+            - TableData
+            - TableMatrixEquation
+            - TableMatrixData
+
+        Notes
+        -----
+        - Table should contain columns for 'Name', 'Formula', and 'State'. Otherwise an error will be raised.
+        - For 'Equation' and 'Data' types, only one component should be provided.
+        - For 'Matrix-Equation' and 'Matrix-Data' types, at least two components should be provided.
+        - The `component_key` parameter determines whether to use the component's name or formula along
         '''
         try:
             # NOTE: detect table type
@@ -1753,23 +1955,67 @@ class ThermoDB(ManageData):
             if isinstance(tb_info_res_, dict):
                 # check
                 if tb_info_res_['Type'] == 'Equation':  # ! equation
-                    # build equation
+
+                    # NOTE: check components
+                    if len(components) != 1:
+                        raise Exception('Only one component required!')
+
+                    if not isinstance(components[0], Component):
+                        raise Exception('Invalid component!')
+
+                    # set component id
+                    component_id_ = None
+                    component_state_ = components[0].state
+                    column_id_ = None
+
+                    # NOTE: set component id and column id
+                    if component_key == 'Name-State':
+                        component_id_ = components[0].name
+                        column_id_ = 'Name'
+                    elif component_key == 'Formula-State':
+                        component_id_ = components[0].formula
+                        column_id_ = 'Formula'
+                    else:
+                        raise Exception('Invalid component_key!')
+
+                    # NOTE: build equation
                     return self.build_equation(
-                        component_name=components[0].name,
+                        component_name=component_id_,
                         databook=databook,
                         table=table,
-                        component_state=components[0].state,
+                        column_name=column_id_,
+                        component_state=component_state_,
                     )
                 elif tb_info_res_['Type'] == 'Data':  # ! data
                     # check
                     if len(components) > 1:
                         raise Exception('Only one component name required!')
-                    # build data
+
+                    if not isinstance(components[0], Component):
+                        raise Exception('Invalid component!')
+
+                    # NOTE: set component id
+                    component_id_ = None
+                    component_state_ = components[0].state
+                    column_id_ = None
+
+                    # NOTE: set component id and column id
+                    if component_key == 'Name-State':
+                        component_id_ = components[0].name
+                        column_id_ = 'Name'
+                    elif component_key == 'Formula-State':
+                        component_id_ = components[0].formula
+                        column_id_ = 'Formula'
+                    else:
+                        raise Exception('Invalid component_key!')
+
+                    # NOTE: build data
                     return self.build_data(
-                        component_name=components[0].name,
+                        component_name=component_id_,
                         databook=databook,
                         table=table,
-                        component_state=components[0].state,
+                        column_name=column_id_,
+                        component_state=component_state_,
                     )
                 elif tb_info_res_['Type'] == 'Matrix-Data':  # ! matrix-data
                     # check
@@ -1811,9 +2057,9 @@ class ThermoDB(ManageData):
         component_name : str
             string of component name (e.g. 'Carbon dioxide')
         databook : int | str
-            databook id or name
+            databook id or name, id is non-zero-based
         table : int | str
-            table id or name
+            table id or name, id is non-zero-based
         column_name : str | list
             column name (e.g. 'Name') | list as ['Name','state']
         query : bool
@@ -1940,14 +2186,19 @@ class ThermoDB(ManageData):
             # table id
             table_id = tb_id + 1
 
+            # check tb_name
+            if tb_name.strip() == '' or tb_name is None:
+                logger.error('Table name not found!')
+
             # SECTION: get data from api
             # ! dataframe and PayLoadType
             component_data = self.get_component_data(
-                component_name,
-                databook_id,
-                table_id,
+                component_name=component_name,
+                databook_id=databook_id,
+                table_id=table_id,
                 column_name=column_name,
-                query=query
+                query=query,
+                component_state=component_state
             )
 
             # check loading state
@@ -1964,9 +2215,8 @@ class ThermoDB(ManageData):
                     # ! check data type
                     _data_type = TransDataC.data_type
                     if _data_type != 'data':
-                        print(
-                            "The selected table contains no data for building data!\
-                            check table id and try again.")
+                        logger.error("The selected table contains no data for building\
+                            data! check table id and try again.")
 
                         raise Exception('Building data failed!')
 
@@ -1974,7 +2224,9 @@ class ThermoDB(ManageData):
                     # * construct template
                     # check eq exists
                     dts = self.data_load(
-                        databook_id, table_id)
+                        databook_id,
+                        table_id
+                    )
 
                     # ! check
                     if dts is not None:
