@@ -9,10 +9,13 @@ from typing import (
     Literal
 )
 # locals
-from ..docs import CustomRef
+from ..loader import CustomRef
 from .builder import TableBuilder
 from .symbols_controller import SymbolController
 
+# NOTE: logger
+logger = logging.getLogger(__name__)
+
 
 class ReferenceChecker:
     """
@@ -1421,7 +1424,10 @@ class ReferenceChecker:
         component_state: str,
         databook_name: str,
         table_name: Optional[str] = None,
-        component_key: Literal['Name-State', 'Formula-State'] = 'Formula-State'
+        component_key: Literal[
+            'Name-State', 'Formula-State'
+        ] = 'Formula-State',
+        ignore_component_state: Optional[bool] = False
     ) -> Dict[str, Union[bool, str]]:
         """
         Check if a component is available in the specified databook.
@@ -1440,6 +1446,8 @@ class ReferenceChecker:
             The name of the table to check, by default None (checks all tables).
         component_key : Literal['Name-State', 'Formula-State'], optional
             The key to use for the components, by default 'Formula-State'.
+        ignore_component_state : Optional[bool], optional
+            Whether to ignore the component state in the check, by default False.
 
         Returns
         -------
@@ -1527,31 +1535,123 @@ class ReferenceChecker:
                         f"Component records for '{component_name}' in table '{table_name}' are missing 'Formula' or 'State'.")
                     continue
 
+                # SECTION: ignore component state if specified
+                if ignore_component_state:
+                    # NOTE: if ignoring state, only check name or formula
+                    if component_key == 'Name-State' and name is not None:
+                        if name.lower().strip() == component_name:
+                            # add
+                            res[table_name] = {
+                                'available': True,
+                                'ignore_state': True
+                            }
+                        else:
+                            # add
+                            res[table_name] = {
+                                'available': False,
+                                'ignore_state': True
+                            }
+
+                        # go to next table
+                        continue
+                    elif component_key == 'Formula-State' and formula is not None:
+                        if formula.lower().strip() == component_formula:
+                            # add
+                            res[table_name] = {
+                                'available': True,
+                                'ignore_state': True
+                            }
+                        else:
+                            # add
+                            res[table_name] = {
+                                'available': False,
+                                'ignore_state': True
+                            }
+
+                        # go to next table
+                        continue
+                    else:
+                        logging.error(
+                            f"Invalid component_key: {component_key}. Must be 'Name-State' or 'Formula-State'.")
+                        continue
+
                 # SECTION: check if component matches the formula and state
                 if component_key == 'Name-State' and (name is not None and state is not None):
+                    # NOTE: check if state symbol is "-"
+                    if state.strip() == '-':
+                        # if state symbol is "-", ignore state in comparison
+                        if name.lower().strip() == component_name:
+                            # add
+                            res[table_name] = {
+                                'available': True,
+                                'ignore_state': False
+                            }
+                        else:
+                            # add
+                            res[table_name] = {
+                                'available': False,
+                                'ignore_state': False
+                            }
+
+                        # go to next table
+                        continue
+
+                    # ! normal comparison
                     if (
                         name.lower().strip() == component_name and
                         state.lower().strip() == component_state
                     ):
                         # add
-                        res[table_name] = True
+                        res[table_name] = {
+                            'available': True,
+                            'ignore_state': False
+                        }
                     else:
                         # add
-                        res[table_name] = False
+                        res[table_name] = {
+                            'available': False,
+                            'ignore_state': False
+                        }
                 elif component_key == 'Formula-State' and (formula is not None and state is not None):
+                    # NOTE: check if state symbol is "-"
+                    if state.strip() == '-':
+                        # if state symbol is "-", ignore state in comparison
+                        if formula.lower().strip() == component_formula:
+                            # add
+                            res[table_name] = {
+                                'available': True,
+                                'ignore_state': True
+                            }
+                        else:
+                            # add
+                            res[table_name] = {
+                                'available': False,
+                                'ignore_state': True
+                            }
+
+                        # go to next table
+                        continue
+
+                    # ! normal comparison
                     if (
                         formula.lower().strip() == component_formula and
                         state.lower().strip() == component_state
                     ):
                         # add
-                        res[table_name] = True
+                        res[table_name] = {
+                            'available': True,
+                            'ignore_state': False
+                        }
                     else:
                         # add
-                        res[table_name] = False
+                        res[table_name] = {
+                            'available': False,
+                            'ignore_state': False
+                        }
                 else:
                     logging.error(
                         f"Invalid component_key: {component_key}. Must be 'Name-State' or 'Formula-State'.")
-                    return {'available': False, 'message': 'Invalid component_key.'}
+                    continue
 
             # res
             return res
@@ -1882,7 +1982,7 @@ class ReferenceChecker:
                             f"Table not found in reference config for '{ref_key}'.")
                         continue
                     # get property name from label (assuming the property name is the same as the label)
-                    prop = table  # This can be modified based on actual requirements
+                    prop = ref_key  # This can be modified based on actual requirements
 
                     # add to reference rules
                     if prop not in reference_rules['EQUATIONS']:
